// Dixit is Copyright (C) 2004-2013 Octavian Procopiuc.
// Homepage: http://dixit.sourceforge.net
// Contact:  Octavian Procopiuc <oprocopiuc@gmail.com>
//
// Use of this source code is governed by the GPLv3
// license that can be found in the LICENSE.GPL3 file,
// or at <http://www.gnu.org/licenses/gpl.html>.
//
// FILENAME:   mainwindow.h
// REVISION:   3
// COMMENTARY: Header file for Qt4 MainWindow class

#ifndef _DIXIT_11_11_4_SRC_MAINWINDOW_H_
#define _DIXIT_11_11_4_SRC_MAINWINDOW_H_

#include "depot.h"
#include <QDir>
#include <QHttp>
#include <QHttpResponseHeader>
#include <QListWidget>
#include <QMainWindow>
#include <QTimer>
#include <QtGui>
#include <QUrl>

QT_BEGIN_NAMESPACE
class QAction;
class QComboBox;
class QDir;
class QHttp;
class QHttpResponseHeader;
class QLabel;
class QLineEdit;
class QListWidget;
class QSplitter;
class QTextBrowser;
class QTimer;
class QUrl;
QT_END_NAMESPACE

enum UpdateErrors {
  UPDATE_ERR_NO_ERROR,
  UPDATE_ERR_NETWORK,
  UPDATE_ERR_SERVER,
  UPDATE_ERR_DB_READ,
  UPDATE_ERR_DB_WRITE,
  UPDATE_ERR_CONTENT,
  UPDATE_ERR_STREAM,
  UPDATE_ERR_TIMEOUT,
  UPDATE_ERR_NOT_DONE,
  UPDATE_ERR_ABORTED
};

class MainWindow : public QMainWindow {
  Q_OBJECT

public:
  MainWindow(void);
  ~MainWindow(void);
  void DbCreateIndex(void);

signals:
  void MoreData(const QHttpResponseHeader &);

private slots:
  void AbortUpdate(void);
  void BackwardInHistory(void);
  void ChangeFont(void);
  void CheckHttpResponseHeader(const QHttpResponseHeader &rh);
  void CloseConnection(bool done);
  void ConnectionTimeout(void);
  void ForwardInHistory(void);
  void GetDexonlineData(const QHttpResponseHeader &rh);
  void Help(void);
  void LinkClicked(const QUrl &link);
  void MakeFontBigger(void);
  void MakeFontSmaller(void);
  void Print(void);
  void Search(void);
  void SetDefaultDictionary(int id);
  void ShowDictionary(int id);
  void Update(void);
  void WordClickedInHistoryBox(const QString &word);

private:
#if QT_VERSION < 0x040500
  void    AppendToByteArray(QByteArray *ba,
                            const char *buf,
                            int sz);
#endif
  void    ClearBrowser(void);
  void    CreateIndependentActions(void);
  void    CreateMainWindow(void);
  void    CreateStatusBar(void);
  void    CreateToolBar(void);
  void    DbFree(char *item);
  void    DbStartIndexing(void);
  void    FindDefinitions(const QString &word);
  void    GenericUpdate(void);
  void    InsertInSearchBox(const QString &word);
  void    LoadSettings(void);
  void    LogInfo(const QString &msg);
  void    ProcessGzipData(QByteArray *iData,
                          char *zData,
                          int zDataSz,
                          bool initOnly);
  void    ProcessWord(const QString &word);
  void    SaveSettings(void);
  void    SetBrowserHtml(const QString &str,
                         bool prs = true);
  void    SetFocusToSearchBox(void);
  bool    DbActive(void);
  bool    DbClose(void);
  bool    DbKeyExists(const char *key);
  bool    DbOpenRead(void);
  bool    DbOpenWrite(void);
  bool    DbPut(const char *key,
                const char *value);
  bool    DbSetup(void);
  int     DbSaveXMLData(const QString &xmlData,
                        UpdateErrors *updateError,
                        bool *abortHttp);
  int     LevenshteinDistance(const QString &source,
                              const QString &target,
                              int maxDist);
  char   *DbGet(const char *key,
                bool *ok);
  QString DbResults(const QString &word);
  QString FormatRoDate(int seconds,
                       bool hour = false);
  QString GetDictionaryName(const QString &shortName);
  QString HtmlizeBrowserEntry(const QString &dbRec);
  QString HtmlizeRawDefinition(const char *rawDef);
  QString LoadFile(const QString &fileName);
  QString ProcessCommands(const QString &cmd);
  QString ReportResults(const QString &word,
                        const QString &distinctWords,
                        const QString &definitions,
                        int stats[]);

  bool          m_activeConnection;
  bool          m_autoUpdate;
  bool          m_logInfo;
  UpdateErrors  m_updateError;
  char          m_dbFileName[128];
  int           m_defaultDictionary;
  int           m_lastUpdateTime;
  int           m_lastUpdateTimePending;
  int           m_newDefCount;
  int           m_newDefCountPending;
  int           m_totalDefinitions;
  QString       m_currentWord;
  QString       m_html;
  QStringList   m_lstAllWords;
  QStringList   m_lstBackwardWords;
  QStringList   m_lstForwardWords;
  QStringList   m_lstNewDefinitions;
  QTimer        m_connectionTimer;

  QComboBox    *w_dictionaryComboBox;
  QLineEdit    *w_searchBox;
  QListWidget  *w_historyListBox;
  QTextBrowser *w_browser;
  QToolBar     *w_toolBar;

  QAction      *a_next;
  QAction      *a_prev;

  QHttp        *m_http;
  DEPOT        *m_dbHandle;

protected:
  QThread      *m_pThread;
};

class CIndexThread : public QThread {
public:
  explicit CIndexThread(MainWindow *w) : QThread(), window(w) {}

protected:
  virtual void run() {
    window->DbCreateIndex();
  }

private:
  MainWindow *window;
};

#endif  // _DIXIT_11_11_4_SRC_MAINWINDOW_H_

// END OF FILE

